package org.bsf.remoteIterator.client.tableModel;

import org.bsf.remoteIterator.client.RemoteIteratorClient;

import javax.swing.table.TableModel;

/**
 * All TableModel that wish to use a RemoteIterator should implement those
 * methods that enable to get informations about the RI.
 */
public interface RemoteIteratorTableModel extends TableModel {
    /**
     * Specifies what RemoteIteratorClient is to be used by the implementing
     * TableModel.
     *
     * @param p_remoteIteratorClient The RemoteIteratorClient to use.
     */
    public void setRemoteIteratorClient( RemoteIteratorClient p_remoteIteratorClient );

    /**
     * Request this model to start loading the data.
     */
    public void startLoadingData();

    /**
     * Request this model to stop loading data.
     */
    public void stopLoadingData();

    /**
     * @return true if the model is loading data in the background, false otherwise.
     */
    public boolean isLoadingData();

    /**
     * Releases the resources (RemoteIterator is first among them but the Threads
     * used (if any) by the implementing model should be released as well.
     */
    public void releaseResources();
}
