/* ******************************************************************************** *
 * Copyright (c) 2002 - 2004 Bright Side Factory. All rights reserved.              *
 *                                                                                  *
 * Redistribution and use in source and binary forms, with or without modification, *
 * are permitted provided that the following conditions are met:                    *
 *                                                                                  *
 * 1. Redistributions of source code must retain the above copyright notice, this   *
 * list of conditions and the following disclaimer.                                 *
 *                                                                                  *
 * 2. Redistributions in binary form must reproduce the above copyright notice,     *
 * this list of conditions and the following disclaimer in the documentation and/or *
 * other materials provided with the distribution.                                  *
 *                                                                                  *
 * 3. The end-user documentation included with the redistribution, if any, must     *
 * include the following acknowledgment: "This product includes software developed  *
 * by the Bright Side Factory (http://www.bs-factory.org/)." Alternately, this      *
 * acknowledgment may appear in the software itself, if and wherever such           *
 * third-party acknowledgments normally appear.                                     *
 *                                                                                  *
 * 4. The names "Bright Side", "BS Framework" and "Bright Side Factory" must not be *
 * used to endorse or promote products derived from this software without prior     *
 * written permission. For written permission, please contact info@bs-factory.org.  *
 *                                                                                  *
 * 5. Products derived from this software may not be called "Bright Side", nor may  *
 * "Bright Side" appear in their name, without prior written permission of the      *
 * Bright Side Factory.                                                             *
 *                                                                                  *
 * THIS SOFTWARE IS PROVIDED ''AS IS'' BY THE COPYRIGHT OWNER AND ANY EXPRESSED OR  *
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF     *
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT *
 * SHALL THE COPYRIGHT OWNER OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,          *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,   *
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,    *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  *
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  *
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS OFTWARE, EVEN IF ADVISED *
 * OF THE POSSIBILITY OF SUCH DAMAGE.                                               *
 *                                                                                  *
 * ================================================================================ *
 *                                                                                  *
 * This software consists of voluntary contributions made by many individuals on    *
 * behalf of the Bright Side Factory. For more information on the Bright Side       *
 * Factory, please see http://www.bs-factory.org.                                   *
 *                                                                                  *
 * ******************************************************************************** */
package org.bsf.remoteIterator.common;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * This class defines the RemoteIterator's result format expected by the client.
 *
 * @see org.bsf.remoteIterator.server.RemoteIteratorBean
 */
public class RemoteIteratorResultTable implements Serializable {
    public static final int RESULT_TABLE_DEFAULT_INITIAL_SIZE = 100;

    // Flags to inform about the state of the ResultTable
    private boolean _isLast;
    private boolean _isFirst;
    private boolean _isBeforeFirst;
    private boolean _isAlreadyLast;

    // Store the rows' values
    private List _rows;

    // Store the rows' indexes
    private List _rowIndexes;

    // *************************************************************************
    // ****************************** Constructor ******************************
    // *************************************************************************

    /**
     * Creates a RemoteIteratorResultTable using the RESULT_TABLE_DEFAULT_INITIAL_SIZE.
     * If you an idea of how many rows will be in this ResultTable you should
     * probably use other constructor. By specifying the number of rows you will
     * prevent ongoing resize of the underlying Lists used to store the rows and
     * the indexes.
     *
     * @see #RemoteIteratorResultTable( int p_size );
     * @see #RESULT_TABLE_DEFAULT_INITIAL_SIZE
     */
    public RemoteIteratorResultTable() {
        _rows = new ArrayList( RESULT_TABLE_DEFAULT_INITIAL_SIZE );
        _rowIndexes = new ArrayList( RESULT_TABLE_DEFAULT_INITIAL_SIZE );
    }

    /**
     * Creates a RemoteIteratorResultTable using the given int as the default
     * size for the underlying Lists. Enables to prevent ongoing resize of the
     * Lists. Should be used if you have an idea of how many rows will be in
     * the Lists.
     *
     * @param p_initialSize The initial size of the underlying Lists.
     *
     * @see #RemoteIteratorResultTable()
     */
    public RemoteIteratorResultTable( int p_initialSize ) {
        _rows = new ArrayList( p_initialSize );
        _rowIndexes = new ArrayList( p_initialSize );
    }

    // *************************************************************************
    // ******************************** Accessors ******************************
    // *************************************************************************

    /**
     * @return the actual number of rows in this RemoteIteratorResultTable. Could
     * be less than the final number if called before all the rows are added to
     * it.
     *
     * @see #addRow
     * @see org.bsf.remoteIterator.server.RemoteIteratorBean
     */
    public int getSize() {
        return _rows.size();
    }

    public boolean isFirst() {
        return _isFirst;
    }

    public void setFirst( boolean p_first ) {
        _isFirst = p_first;
    }

    public boolean isLast() {
        return _isLast;
    }

    public void setLast( boolean p_last ) {
        _isLast = p_last;
    }

    public boolean isAlreadyLast() {
        return _isAlreadyLast;
    }

    public void setAlreadyLast( boolean p_alreadyLast ) {
        _isAlreadyLast = p_alreadyLast;
    }

    public boolean isBeforeFirst() {
        return _isBeforeFirst;
    }

    public void setBeforeFirst( boolean p_beforeFirst ) {
        _isBeforeFirst = p_beforeFirst;
    }

    public List getRows() {
        return _rows;
    }

    public List getRowIndexes() {
        return _rowIndexes;
    }

    /**
     * adds the row (an arrayList) in the list of result.
     * Used by the RI.
     * @param p_rows
     */
    public void addRow( List p_rows, Integer p_rowsPosition ) {
        _rows.add( p_rows );
        _rowIndexes.add( p_rowsPosition );
    }

}
