/* ******************************************************************************** *
 * Copyright (c) 2002 - 2004 Bright Side Factory. All rights reserved.              *
 *                                                                                  *
 * Redistribution and use in source and binary forms, with or without modification, *
 * are permitted provided that the following conditions are met:                    *
 *                                                                                  *
 * 1. Redistributions of source code must retain the above copyright notice, this   *
 * list of conditions and the following disclaimer.                                 *
 *                                                                                  *
 * 2. Redistributions in binary form must reproduce the above copyright notice,     *
 * this list of conditions and the following disclaimer in the documentation and/or *
 * other materials provided with the distribution.                                  *
 *                                                                                  *
 * 3. The end-user documentation included with the redistribution, if any, must     *
 * include the following acknowledgment: "This product includes software developed  *
 * by the Bright Side Factory (http://www.bs-factory.org/)." Alternately, this      *
 * acknowledgment may appear in the software itself, if and wherever such           *
 * third-party acknowledgments normally appear.                                     *
 *                                                                                  *
 * 4. The names "Bright Side", "BS Framework" and "Bright Side Factory" must not be *
 * used to endorse or promote products derived from this software without prior     *
 * written permission. For written permission, please contact info@bs-factory.org.  *
 *                                                                                  *
 * 5. Products derived from this software may not be called "Bright Side", nor may  *
 * "Bright Side" appear in their name, without prior written permission of the      *
 * Bright Side Factory.                                                             *
 *                                                                                  *
 * THIS SOFTWARE IS PROVIDED ''AS IS'' BY THE COPYRIGHT OWNER AND ANY EXPRESSED OR  *
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF     *
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT *
 * SHALL THE COPYRIGHT OWNER OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,          *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,   *
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,    *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  *
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  *
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS OFTWARE, EVEN IF ADVISED *
 * OF THE POSSIBILITY OF SUCH DAMAGE.                                               *
 *                                                                                  *
 * ================================================================================ *
 *                                                                                  *
 * This software consists of voluntary contributions made by many individuals on    *
 * behalf of the Bright Side Factory. For more information on the Bright Side       *
 * Factory, please see http://www.bs-factory.org.                                   *
 *                                                                                  *
 * ******************************************************************************** */
package org.bsf.remoteIterator.server;

import org.bsf.commons.ejb.SessionAdapterBean;

import javax.ejb.CreateException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * This stateless ejb provides access to the remote iterator features.
 *
 * @see org.bsf.remoteIterator.server.RemoteIteratorBean
 *
 *  @ejb:bean       type="Stateless"
 *                  name="RemoteIteratorService"
 *                  jndi-name="ejb/RemoteIteratorService"
 *                  local-jndi-name="ejb/RemoteIteratorServiceLocal"
 *                  view-type="both"
 *
 *  @ejb:home       extends="javax.ejb.EJBHome"
 *                  local-extends="javax.ejb.EJBLocalHome"
 *
 *  @ejb:interface  extends="javax.ejb.EJBObject"
 *                  local-extends="javax.ejb.EJBLocalObject"
 *
 *  @ejb:ejb-ref    ejb-name="RemoteIterator"
 *                  view-type="remote"
 *
 *  @ejb:transaction type="Supports"
 *
 *  @jonas.bean         ejb-name="RemoteIteratorService"
 *                      jndi-name="ejb/RemoteIteratorService"
 */
public class RemoteIteratorServiceBean extends SessionAdapterBean {
    private RemoteIteratorHome _remoteIteratorHome = null;

    /**
     * @ejb:interface-method
     */
    public RemoteIterator getRemoteIterator( String p_query ) {
        RemoteIterator remoteIterator = null;

        try {
            remoteIterator = _remoteIteratorHome.create( p_query );
        } catch( Exception e ) {
            // A RemoteException or a CreateException...
            logError( "Unable to create ri with sql: " + p_query );

            handleExceptionAsSystemException( e );
        }

        return remoteIterator;
    }

    // *************************************************************************
    // ******************** SesssionAdapterBean Methods ************************
    // *************************************************************************

    public void ejbCreate() throws CreateException {
        logDebug( "ejbCreate() -> Retrieving the home(s)..." );

        // Retrieves the local home of the RemoteIteratorBean and keeps it for further use
        try {
            InitialContext ic = new InitialContext();
            _remoteIteratorHome = (RemoteIteratorHome) ic.lookup( "java:comp/env/ejb/RemoteIterator" );
        } catch( NamingException e ) {
            handleExceptionAsSystemException( e );
        }
    }
}

