package org.bsf.remoteIterator;

import org.bsf.remoteIterator.client.RemoteIteratorClient;
import org.bsf.remoteIterator.client.tableModel.DefaultRemoteIteratorTableModel;
import org.bsf.remoteIterator.server.RemoteIteratorService;
import org.bsf.remoteIterator.server.RemoteIteratorServiceHome;
import org.bsf.remoting.EJBDefinition;
import org.bsf.remoting.http.HttpServiceFactory;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;
import java.awt.BorderLayout;
import java.awt.HeadlessException;
import java.util.Properties;

/**
 * BasicRemoteIteratorFrame
 */
public class BasicRemoteIteratorFrame extends JFrame {
    // Set to true to use the remoting, false for rmi
    private boolean _mustUseRemoting = true;

    private RemoteIteratorService _riService = null;
    private HttpServiceFactory _factory = new HttpServiceFactory();

    private static final String SQL_STATEMENT = "select * from " + RITestDatabaseManager.TABLE_NAME;

    EJBDefinition REMOTE_ITERATOR_SERVICE_DEFINITION = new EJBDefinition( "ejb/RemoteIteratorService",
                                                                          "org.bsf.remoteIterator.server.RemoteIteratorServiceHome",
                                                                          "org.bsf.remoteIterator.server.RemoteIteratorService" );

    public BasicRemoteIteratorFrame() throws HeadlessException {
        super( "Basic Frame containing a Table using the DefaultremoteIteratorTableModel" );
        setBounds( 100, 100, 650, 500 );

        DefaultRemoteIteratorTableModel tableModel = new DefaultRemoteIteratorTableModel();

        DefaultTableColumnModel columnModel = new DefaultTableColumnModel();
        TableColumn aColumn = new TableColumn();
        aColumn.setHeaderValue( "OID" );
        columnModel.addColumn( aColumn );
        aColumn = new TableColumn();
        aColumn.setHeaderValue( "Label" );
        columnModel.addColumn( aColumn );

        JTable myTable = new JTable( tableModel, columnModel );

        getContentPane().setLayout( new BorderLayout() );
        getContentPane().add( new JScrollPane( myTable ), BorderLayout.CENTER );

        tableModel.setRemoteIteratorClient( getRemoteIteratorClient( SQL_STATEMENT ) );
        tableModel.setInitialBlockSize( 25 );
        tableModel.setDefaultBlockSize( 500 );
        tableModel.setDelayBetweenRetrieval( 250 );

        // We start loading the data
        tableModel.startLoadingData();
    }

    protected RemoteIteratorClient getRemoteIteratorClient( String p_statement ) {
        RemoteIteratorClient riClient = null;

        try {
            if ( _riService == null ) {
                if ( _mustUseRemoting ) {
                    // The port to use, on the server, to communicate (default is 8080)
                    _factory.setPort( 8080 );
                    // The location of the server on which the server is deployed
                    _factory.setHost( "localhost" );
                    // The context of the remoting (in the ear def)
                    _factory.setServerContext( "testRI" );

                    _riService = (RemoteIteratorService) _factory.getService( REMOTE_ITERATOR_SERVICE_DEFINITION );
                } else {
                    Context jndiContext = getInitialContext();
                    _riService = ( (RemoteIteratorServiceHome) jndiContext.lookup( "ejb/RemoteIteratorService" ) ).create();
                }
            }

            riClient = new RemoteIteratorClient( _riService.getRemoteIterator( p_statement ) );
        } catch( Exception e ) {
            // We display the stack trace
            e.printStackTrace();

            // And exit
            System.exit( 0 );
        }

        return riClient;
    }

    private Context getInitialContext() throws NamingException {
        Properties properties = new Properties();
        properties.put( "java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory" );
        properties.put( "java.naming.provider.url", "jnp://localhost:1099" );
        properties.put( "java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces" );

        return new InitialContext( properties );
    }

    public static void main( String[] args ) {
        BasicRemoteIteratorFrame myFrame = new BasicRemoteIteratorFrame();
        myFrame.setDefaultCloseOperation( EXIT_ON_CLOSE );
        myFrame.show();
    }
}
