/* ******************************************************************************** *
 * Copyright (c) 2002 - 2004 Bright Side Factory. All rights reserved.              *
 *                                                                                  *
 * Redistribution and use in source and binary forms, with or without modification, *
 * are permitted provided that the following conditions are met:                    *
 *                                                                                  *
 * 1. Redistributions of source code must retain the above copyright notice, this   *
 * list of conditions and the following disclaimer.                                 *
 *                                                                                  *
 * 2. Redistributions in binary form must reproduce the above copyright notice,     *
 * this list of conditions and the following disclaimer in the documentation and/or *
 * other materials provided with the distribution.                                  *
 *                                                                                  *
 * 3. The end-user documentation included with the redistribution, if any, must     *
 * include the following acknowledgment: "This product includes software developed  *
 * by the Bright Side Factory (http://www.bs-factory.org/)." Alternately, this      *
 * acknowledgment may appear in the software itself, if and wherever such           *
 * third-party acknowledgments normally appear.                                     *
 *                                                                                  *
 * 4. The names "Bright Side", "BS Framework" and "Bright Side Factory" must not be *
 * used to endorse or promote products derived from this software without prior     *
 * written permission. For written permission, please contact info@bs-factory.org.  *
 *                                                                                  *
 * 5. Products derived from this software may not be called "Bright Side", nor may  *
 * "Bright Side" appear in their name, without prior written permission of the      *
 * Bright Side Factory.                                                             *
 *                                                                                  *
 * THIS SOFTWARE IS PROVIDED ''AS IS'' BY THE COPYRIGHT OWNER AND ANY EXPRESSED OR  *
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF     *
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT *
 * SHALL THE COPYRIGHT OWNER OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,          *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,   *
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,    *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  *
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  *
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS OFTWARE, EVEN IF ADVISED *
 * OF THE POSSIBILITY OF SUCH DAMAGE.                                               *
 *                                                                                  *
 * ================================================================================ *
 *                                                                                  *
 * This software consists of voluntary contributions made by many individuals on    *
 * behalf of the Bright Side Factory. For more information on the Bright Side       *
 * Factory, please see http://www.bs-factory.org.                                   *
 *                                                                                  *
 * ******************************************************************************** */
package org.bsf.remoteIterator;

import java.sql.*;

/**
 * Enables the creation of a table in the configured DB to test the RI. The
 * TestRemoteIterator is using the TableName configured in this Helping class.
 */
public class RITestDatabaseManager {
    private Connection _connection = null;
    private Statement _sqlStatement = null;
    private ResultSet _resultSet = null;

    // DB CNX PROPERTIES
    private String DRIVER = "org.gjt.mm.mysql.Driver";
    private String URL = "jdbc:mysql://localhost:3306/bookstore";
    private String USER = "bookstore";
    private String PASSWORD = "bookstore";

    // DB VALUES
    public static int NB_RECORDS = 10000;
    public static String TABLE_NAME = "test_ri_table";

    private String REMOVE_STATEMENT
            = "DROP TABLE " + TABLE_NAME;

    private String CREATE_STATEMENT
            = "CREATE TABLE " + TABLE_NAME + " (oid INT PRIMARY KEY NOT NULL, label VARCHAR(20) NOT NULL)";

    private String INSERT_STATEMENT
            = "INSERT INTO " + TABLE_NAME + " VALUES (%OID%, %LABEL%)";

    public RITestDatabaseManager() {
        super();

        try {
            System.setProperty( "jdbc.drivers", DRIVER );
            _connection = DriverManager.getConnection( URL, USER, PASSWORD );
        } catch( SQLException sqlException ) {
            System.out.println( "Unable to reach the DB with the parameters: " );
            System.out.println( "Driver   => " + DRIVER );
            System.out.println( "URL      => " + URL );
            System.out.println( "USER     => " + USER );
            System.out.println( "PASSWORD => " + PASSWORD + "\n" );

            sqlException.printStackTrace();

            System.exit( 0 );
        }
    }

    // *************************************************************************
    // ************************** Actions methods ******************************
    // *************************************************************************

    public void removeTable() {
        System.out.print( "Removing the old " + TABLE_NAME + " table if needed... " );

        try {
            _sqlStatement = _connection.createStatement();
            _sqlStatement.execute( REMOVE_STATEMENT );
        } catch( SQLException sqlException ) {
            // Nothing to do (probably no such table...
        } finally {
            showDone();
        }

        // We close the statement here cause we might want to go on (with creation, population...)
        try {
            _sqlStatement.close();
            _sqlStatement = null;
        } catch( SQLException e ) {
            // Who cares...
        }
    }

    public void createTable() throws SQLException {
        System.out.print( "Creating the table " + TABLE_NAME + " " );

        _sqlStatement = _connection.createStatement();

        _sqlStatement.execute( CREATE_STATEMENT );

        showDone();
    }

    public void populateTable() throws SQLException {
        System.out.print( "Populating the " + TABLE_NAME + " table with " + NB_RECORDS + " rows... " );

        _sqlStatement = _connection.createStatement();

        for ( int index = 0 ; index < NB_RECORDS ; index++ ) {
            String sql = INSERT_STATEMENT.replaceAll( "%OID%", String.valueOf( index ) );
            sql = sql.replaceAll( "%LABEL%", "'Label " + index + "'" );

            _sqlStatement.execute( sql );
        }

        showDone();
    }

    public void releaseRessources() {
        try {
            if ( _resultSet != null ) {
                System.out.print( "Closing the ResultSet " );
                _resultSet.close();
                showDone();
            }
        } catch( SQLException e ) {
            showFaillure();
        }

        try {
            if ( _sqlStatement != null ) {
                System.out.print( "Closing the Statement " );
                _sqlStatement.close();
                showDone();
            }
        } catch( SQLException e ) {
            showFaillure();
        }

        try {
            if ( _connection != null ) {
                System.out.print( "Closing the Connection " );
                _connection.close();
                showDone();
            }
        } catch( SQLException e ) {
            showFaillure();
        }
    }

    // *************************************************************************
    // ************************** Utility methods ******************************
    // *************************************************************************

    private void showDone() {
        System.out.println( "[DONE]" );
    }

    private void showFaillure() {
        System.out.println( "[FAILLURE]" );
    }

    // *************************************************************************
    // ***************************** Main method *******************************
    // *************************************************************************

    public static void main( String[] args ) {
        if ( args.length == 0 || args.length > 1 ) {
            System.out.println( "RITestDatabaseManager usage:" );
            System.out.println( "   => (...) RITestDatabaseManager [OPTION]" );
            System.out.println( "\nPossible values for OPTION are:" );
            System.out.println( "   -removeTestTable => Removes the table for the test" );
            System.out.println( "   -createTestTable => Creates and populates the table for the test" );

            // We quit, no valid parameters...
            System.exit( 0 );
        }

        RITestDatabaseManager dbManager = new RITestDatabaseManager();

        if ( "-removeTestTable".equalsIgnoreCase( args[ 0 ] ) ) {
            dbManager.removeTable();
        } else if ( "-createTestTable".equalsIgnoreCase( args[ 0 ] ) ) {
            dbManager.removeTable();

            try {
                dbManager.createTable();
                dbManager.populateTable();
            } catch( SQLException sqlException ) {
                dbManager.showFaillure();

                // And display the stacktrace
                System.out.println( "\n" );
                sqlException.printStackTrace();
            }
        }

        // We release the ressources
        dbManager.releaseRessources();
    }
}
