/* ******************************************************************************** *
 * Copyright (c) 2002 - 2004 Bright Side Factory. All rights reserved.              *
 *                                                                                  *
 * Redistribution and use in source and binary forms, with or without modification, *
 * are permitted provided that the following conditions are met:                    *
 *                                                                                  *
 * 1. Redistributions of source code must retain the above copyright notice, this   *
 * list of conditions and the following disclaimer.                                 *
 *                                                                                  *
 * 2. Redistributions in binary form must reproduce the above copyright notice,     *
 * this list of conditions and the following disclaimer in the documentation and/or *
 * other materials provided with the distribution.                                  *
 *                                                                                  *
 * 3. The end-user documentation included with the redistribution, if any, must     *
 * include the following acknowledgment: "This product includes software developed  *
 * by the Bright Side Factory (http://www.bs-factory.org/)." Alternately, this      *
 * acknowledgment may appear in the software itself, if and wherever such           *
 * third-party acknowledgments normally appear.                                     *
 *                                                                                  *
 * 4. The names "Bright Side", "BS Framework" and "Bright Side Factory" must not be *
 * used to endorse or promote products derived from this software without prior     *
 * written permission. For written permission, please contact info@bs-factory.org.  *
 *                                                                                  *
 * 5. Products derived from this software may not be called "Bright Side", nor may  *
 * "Bright Side" appear in their name, without prior written permission of the      *
 * Bright Side Factory.                                                             *
 *                                                                                  *
 * THIS SOFTWARE IS PROVIDED ''AS IS'' BY THE COPYRIGHT OWNER AND ANY EXPRESSED OR  *
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF     *
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT *
 * SHALL THE COPYRIGHT OWNER OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,          *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,   *
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,    *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  *
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  *
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS OFTWARE, EVEN IF ADVISED *
 * OF THE POSSIBILITY OF SUCH DAMAGE.                                               *
 *                                                                                  *
 * ================================================================================ *
 *                                                                                  *
 * This software consists of voluntary contributions made by many individuals on    *
 * behalf of the Bright Side Factory. For more information on the Bright Side       *
 * Factory, please see http://www.bs-factory.org.                                   *
 *                                                                                  *
 * ******************************************************************************** */
package org.bsf.remoteIterator;

import junit.framework.TestCase;
import org.bsf.remoteIterator.client.RemoteIteratorClient;
import org.bsf.remoteIterator.common.ColumnMetadata;
import org.bsf.remoteIterator.server.RemoteIteratorService;
import org.bsf.remoteIterator.server.RemoteIteratorServiceHome;
import org.bsf.remoting.EJBDefinition;
import org.bsf.remoting.http.HttpServiceFactory;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

/**
 * TestRemoteIterator
 */
public class TestRemoteIterator extends TestCase {
    // Set to true to use the remoting, false for rmi
    private boolean _mustUseRemoting = true;

    private RemoteIteratorClient _riClient = null;
    private RemoteIteratorService _riService = null;
    private HttpServiceFactory _factory = new HttpServiceFactory();

    EJBDefinition REMOTE_ITERATOR_SERVICE_DEFINITION = new EJBDefinition( "ejb/RemoteIteratorService",
                                                                          "org.bsf.remoteIterator.server.RemoteIteratorServiceHome",
                                                                          "org.bsf.remoteIterator.server.RemoteIteratorService" );

    private static final String SQL_STATEMENT = "select * from " + RITestDatabaseManager.TABLE_NAME;

    public TestRemoteIterator( String s ) {
        super( s );

        try {
            if ( _mustUseRemoting ) {
                // The port to use, on the server, to communicate (default is 8080)
                _factory.setPort( 8080 );
                // The location of the server on which the server is deployed
                _factory.setHost( "localhost" );
                // The context of the remoting (in the ear def)
                _factory.setServerContext( "testRI" );

                _riService = (RemoteIteratorService) _factory.getService( REMOTE_ITERATOR_SERVICE_DEFINITION );
            } else {
                Context jndiContext = getInitialContext();
                _riService = ( (RemoteIteratorServiceHome) jndiContext.lookup( "ejb/RemoteIteratorService" ) ).create();
            }
        } catch( Exception e ) {
            // We display the stack trace
            e.printStackTrace();

            // And exit
            System.exit( 0 );
        }
    }

    private Context getInitialContext() throws NamingException {
        Properties properties = new Properties();
        properties.put( "java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory" );
        properties.put( "java.naming.provider.url", "jnp://localhost:1099" );
        properties.put( "java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces" );

        return new InitialContext( properties );
    }

    // *************************************************************************
    // ******************************* Tests ***********************************
    // *************************************************************************

    /**
     * Testing the results of a call to getColumnMetaData()...
     */
    public void testRemoteIteratorMetaData() throws Exception {
        System.out.print( "\nRunning testRemoteIteratorMetaData" );

        _riClient = getRemoteIteratorClient( SQL_STATEMENT );

        List columnsMetaData = _riClient.getColumnMetatData();

        for ( int index = 0 ; index < columnsMetaData.size() ; index++ ) {
            ColumnMetadata columnMetaData = (ColumnMetadata) columnsMetaData.get( index );

            String tableName = columnMetaData.getTableName();
            String columnName = columnMetaData.getColumnName();
            String columnClassName = columnMetaData.getColumnClassName();

            String expectedColumnName = ( index == 0 ) ? "OID" : "LABEL";
            String expectedColumnClassName = ( index == 0 ) ? "java.lang.Integer" : "java.lang.String";

            if ( !RITestDatabaseManager.TABLE_NAME.equalsIgnoreCase( tableName ) ) {
                fail( "\nInvalid table name... Expected " + RITestDatabaseManager.TABLE_NAME +
                      " found " + tableName );
            }

            if ( !expectedColumnName.equalsIgnoreCase( columnName ) ) {
                fail( "\nInvalid column name... Expected " + expectedColumnName + " found " + columnName );
            }

            if ( !expectedColumnClassName.equalsIgnoreCase( columnClassName ) ) {
                fail( "\nInvalid column name... Expected " + expectedColumnName + " found " + columnName );
            }
        }

        System.out.println( " => SUCCESS !!!" );
    }

    /**
     * Testing the results of a call to getRowCount()...
     */
    public void testRemoteIteratorRecordsCount() throws Exception {
        System.out.print( "\nRunning testRemoteIteratorRecordsCount" );

        _riClient = getRemoteIteratorClient( SQL_STATEMENT );

        int recordsCount = _riClient.getRecordCount();

        if ( RITestDatabaseManager.NB_RECORDS != recordsCount ) {
            fail( "\nInvalid records count... Expected " + RITestDatabaseManager.NB_RECORDS + " found " + recordsCount );
        }

        System.out.println( " => SUCCESS !!!" );
    }

    /**
     * The default table as rows whose first column (its oid) is the row index...
     * The difference between two following rows should always be one... If not,
     * that means that the next are missing some rows :o(
     */
    public void testRemoteIteratorRetrieval() throws Exception {
        System.out.print( "\nRunning testRemoteIteratorRetrieval" );

        _riClient = getRemoteIteratorClient( SQL_STATEMENT );

        int previousRowOID = -1;

        while ( !_riClient.isLast() ) {
            Iterator iterator = _riClient.next( 250 );

            while ( iterator.hasNext() ) {
                List row = (List) iterator.next();

                if ( ( (Number) row.get( 0 ) ).intValue() - previousRowOID != 1 ) {
                    fail( "\nError in interval retrieval..." );
                } else {
                    previousRowOID = ( (Number) row.get( 0 ) ).intValue();
                }
            }
        }

        System.out.println( " => SUCCESS !!!" );
    }

    /**
     * The default table as rows whose first column (its oid) is the row index...
     * Checks if the cursor is well moved.
     */
    public void testRemoteIteratorCursor() throws Exception {
        System.out.print( "\nRunning testRemoteIteratorCursor" );

        int nbRowsToMove = 0;
        int expectedRowOID = 0;

        _riClient = getRemoteIteratorClient( SQL_STATEMENT );

        nbRowsToMove = 50;
        List row = (List) _riClient.next( nbRowsToMove ).next();

        if ( ( (Number) row.get( 0 ) ).intValue() != expectedRowOID ) {
            fail( "\nError in next retrieval... Found: " + (Number) row.get( 0 ) + " expexted: " + expectedRowOID );
        }

        expectedRowOID += nbRowsToMove;
        nbRowsToMove = 25;
        row = (List) _riClient.next( nbRowsToMove ).next();

        if ( ( (Number) row.get( 0 ) ).intValue() != expectedRowOID ) {
            fail( "\nError in next retrieval... Found: " + (Number) row.get( 0 ) + " expexted: " + expectedRowOID );
        }

        expectedRowOID += nbRowsToMove;
        nbRowsToMove = 5;
        row = (List) _riClient.next( nbRowsToMove ).next();

        if ( ( (Number) row.get( 0 ) ).intValue() != expectedRowOID ) {
            fail( "\nError in next retrieval... Found: " + (Number) row.get( 0 ) + " expexted: " + expectedRowOID );
        }

        nbRowsToMove = 10;
        expectedRowOID -= nbRowsToMove; // Begining of next block
        row = (List) _riClient.previous( nbRowsToMove ).next();

        if ( ( (Number) row.get( 0 ) ).intValue() != expectedRowOID ) {
            fail( "\nError in previous retrieval... Found: " + (Number) row.get( 0 ) + " expexted: " + expectedRowOID );
        }

        nbRowsToMove = 1000; // Larger than the actual row
        expectedRowOID = 0; // Begining of the cursor
        row = (List) _riClient.previous( nbRowsToMove ).next();

        if ( ( (Number) row.get( 0 ) ).intValue() != expectedRowOID ) {
            fail( "\nError in previous retrieval... Found: " + (Number) row.get( 0 ) + " expexted: " + expectedRowOID );
        }

        expectedRowOID = 100;
        row = (List) _riClient.absolute( 100 ).next();

        if ( ( (Number) row.get( 0 ) ).intValue() != expectedRowOID ) {
            fail( "\nError in absolute retrieval... Found: " + (Number) row.get( 0 ) + " expexted: " + expectedRowOID );
        }

        expectedRowOID = 2750;
        row = (List) _riClient.absolute( 2750 ).next();

        if ( ( (Number) row.get( 0 ) ).intValue() != expectedRowOID ) {
            fail( "\nError in absolute retrieval... Found: " + (Number) row.get( 0 ) + " expexted: " + expectedRowOID );
        }

        System.out.println( " => SUCCESS !!!" );
    }

    // *************************************************************************
    // ************************** Utility methods ******************************
    // *************************************************************************

    protected RemoteIteratorClient getRemoteIteratorClient( String p_statement ) {
        RemoteIteratorClient riClient = null;

        try {
            riClient = new RemoteIteratorClient( _riService.getRemoteIterator( p_statement ) );
        } catch( java.rmi.RemoteException e ) {
            fail( "Check the SQL Statement... " + p_statement );
        }

        return riClient;
    }

    /**
     * Called after each test... We remove the RI (if needed).
     *
     * @throws Exception
     */
    protected void tearDown() throws Exception {
        super.tearDown();

        if ( _riClient != null && _riClient.hasRemoteReference() ) {
            _riClient.remove();
        }
    }
}
